package com.qx.didi.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.api.R;
import com.qx.didi.dto.ApiDTO;
import com.qx.didi.dto.Response;
import com.qx.didi.dto.SohResultDto;
import com.qx.didi.entity.*;
import com.qx.didi.mapper.SohResultMapper;
import com.qx.didi.param.QueryParam;
import com.qx.didi.service.impl.*;
import com.qx.didi.util.ApiRuntimeException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.SimpleDateFormat;
import java.util.*;

@Slf4j
@Api(tags = {"数据显示"})
@RestController
@RequestMapping("/showData")
public class ShowDataController {

    @Autowired
    ResultInfoServiceImpl resultInfoService;

    @Autowired
    ResultAlarmInfoServiceImpl resultAlarmInfoService;
    @Autowired
    ResultInnershortServiceImpl resultInnershortService;
    @Autowired
    ResultOriginDataServiceImpl resultOriginDataService;
    @Autowired
    ResultVoltOutlierServiceImpl resultVoltOutlierService;
    @Autowired
    ResultVoltSocServiceImpl resultVoltSocService;
    @Autowired
    ResultVoltdiffOutlierServiceImpl resultVoltdiffOutlierService;
    @Autowired
    SohResultMapper sohResultMapper;

    @ApiOperation(value = "获取汇总数据")
    @PostMapping("/getTotal")
    public ApiDTO getTotal(@RequestBody QueryParam param) {
        List<Response> responses = new ArrayList<>();
        List<String> sns = new ArrayList<>();
        QueryWrapper<ResultOriginData> resultOriginDataQueryWrapper = new QueryWrapper<>();
        if (StringUtils.hasLength(param.getSn())) {
            resultOriginDataQueryWrapper.like("device_id", param.getSn());
        }
        if (param.getTableId() != null) {
            resultOriginDataQueryWrapper.eq("type", param.getTableId());
        }
        if (param.getBatchNum() != null) {
            resultOriginDataQueryWrapper.like("batch_num", param.getBatchNum());
        }
        List<ResultOriginData> resultOriginDataList = resultOriginDataService.list(resultOriginDataQueryWrapper);
        resultOriginDataList.forEach(p -> sns.add(p.getDeviceId()));
        if (!CollectionUtils.isEmpty(sns)) {
            List<ResultInfo> resultInfos = resultInfoService.list(new QueryWrapper<ResultInfo>().in("sn", sns).orderBy(true, false, "status"));
            List<String> sns2 = new ArrayList<>();
            resultInfos.forEach(p -> sns2.add(p.getSn()));
            sns2.forEach(p -> {
                QueryParam param1 = new QueryParam();
                param1.setSn(p);

                ApiDTO apiDTO = getRight(param1);
                responses.add((Response) apiDTO.getData());
            });
        }
        return ApiDTO.ok(responses);
    }


    @ApiOperation(value = "获取右侧数据")
    @PostMapping("/getRight")
    public ApiDTO getRight(@RequestBody QueryParam param) {

        try {
            log.info(param.getSn() + "正在获取右侧数据");
            Response response = new Response();
            response.setSn(param.getSn());
            ResultInfo resultInfo = new ResultInfo();
            // 诊断信息

            resultInfo = resultInfoService.getOne(new QueryWrapper<ResultInfo>().eq("sn", param.getSn()));
            if (resultInfo != null) {
                if (resultInfo.getSafety() != null) {
                    resultInfo.setSafety(resultInfo.getSafety().replace("；", ";"));
                    response.setSafety(Arrays.asList(resultInfo.getSafety().split(";")));
                }
                if (resultInfo.getSafetyAdvice() != null) {
                    resultInfo.setSafetyAdvice(resultInfo.getSafetyAdvice().replace("；", ";"));
                    response.setSafetyAdvice(Arrays.asList(resultInfo.getSafetyAdvice().split(";")));
                }

                if (resultInfo.getSafetyText() != null) {
                    resultInfo.setSafetyText(resultInfo.getSafetyText().replace("；", ";"));
                    response.setSafetyText(Arrays.asList(resultInfo.getSafetyText().split(";")));
                }

                if (resultInfo.getSafetyAdvice() != null) {
                    resultInfo.setSafetyAdvice(resultInfo.getSafetyAdvice().replace("；", ";"));
                    response.setSafetyAdvice(Arrays.asList(resultInfo.getSafetyAdvice().split(";")));
                }
                if (resultInfo.getPerform() != null) {
                    resultInfo.setPerform(resultInfo.getPerform().replace("；", ";"));
                    response.setPerform(Arrays.asList(resultInfo.getPerform().split(";")));
                }
                if (resultInfo.getPerformAdvice() != null) {
                    resultInfo.setPerformAdvice(resultInfo.getPerformAdvice().replace("；", ";"));
                    response.setPerformAdvice(Arrays.asList(resultInfo.getPerformAdvice().split(";")));
                }
                response.setReason(resultInfo.getReason());
                response.setAlarmCell(resultInfo.getLocation());

                response.setRiskNotation(resultInfo.getRiskNotation());
                if (resultInfo.getAlarmTime() != null) {
                    response.setAlarmTime(new SimpleDateFormat("yyyy-MM-dd").format(resultInfo.getAlarmTime()));

                }
                response.setScore(resultInfo.getScore());
                response.setSoh(resultInfo.getSoh());
                response.setCapUniform(resultInfo.getCapUniform());
                response.setSocUniform(resultInfo.getSocUniform());
                response.setInnershortIndex(resultInfo.getInnershortIndex());
                response.setSohUniform(resultInfo.getSohUniform());
                response.setStatus(resultInfo.getStatus());
                switch (resultInfo.getStatus()){
                    case 1: response.setStatusText("安全(S)");break;
                    case 2: response.setStatusText("预警(W)");break;
                    case 3: response.setStatusText("危险(D)");break;
                    case 4: response.setStatusText("热失控(T)");break;
                    default:response.setStatusText("未知");
                }
            }


//            if (param.getTableId().equals(1)) {
//                if (resultInfo.getAlarmTime() != null) {
//                    response.setAlarmTime(new SimpleDateFormat("yyyy-MM-dd").format(resultInfo.getAlarmTime()));
//                }
//            }
//
//            if (param.getTableId().equals(2)) {
//                response.setScore(resultInfo.getScore());
//                response.setSoh(resultInfo.getSoh());
//                response.setCapUniform(resultInfo.getCapUniform());
//                response.setSocUniform(resultInfo.getSocUniform());
//                response.setInnershortIndex(resultInfo.getInnershortIndex());
//                response.setSohUniform(resultInfo.getSohUniform());
//            }

            // 当前信息
            ResultOriginData resultOriginData = resultOriginDataService.getOne(new QueryWrapper<ResultOriginData>().eq("device_id", param.getSn()));
            response.setTableId(resultOriginData.getType());
            response.setTime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(resultOriginData.getDate()));
            response.setSoc(resultOriginData.getSoc());
            response.setVoltage(resultOriginData.getVoltage());
            response.setCurrent(resultOriginData.getCurrent());
            List<Float> voltages = Arrays.asList(resultOriginData.getCellvolt1(), resultOriginData.getCellvolt2(), resultOriginData.getCellvolt3(),
                    resultOriginData.getCellvolt4(), resultOriginData.getCellvolt5(), resultOriginData.getCellvolt6(), resultOriginData.getCellvolt7(), resultOriginData.getCellvolt8(),
                    resultOriginData.getCellvolt9());
            response.setMaxVolt(Collections.max(voltages));
            response.setMaxVoltIndex("单体电芯" + voltages.indexOf(Collections.max(voltages)));
            response.setMinVolt(Collections.min(voltages));
            response.setMinVoltIndex("单体电芯" + voltages.indexOf(Collections.min(voltages)));
            response.setCellTemp(resultOriginData.getCelltemp());

            // 报警信息
            List<ResultAlarmInfo> resultAlarmInfo = resultAlarmInfoService.list(new QueryWrapper<ResultAlarmInfo>().eq("sn", param.getSn()));
            if (resultAlarmInfo != null && resultAlarmInfo.size() > 0) {
                for (ResultAlarmInfo r : resultAlarmInfo) {
                    if (r.getType().equals("电压离群报警")) {
                        response.setVoltAlarmTime(new SimpleDateFormat("yyyy-MM-dd").format(r.getAlarmtime()));
                        response.setVoltValue(r.getValue());
                        response.setVoltValueDiff(r.getValuediff());
                        response.setVoltThreshold(new ArrayList<Object>() {{
                            add(r.getThresholdDown());
                            add(r.getThresholdUp());
                        }});
                        response.setVoltLevel(r.getLevel());
                    } else if (r.getType().equals("压差离群报警")) {
                        response.setVoltDiffAlarmTime(new SimpleDateFormat("yyyy-MM-dd").format(r.getAlarmtime()));
                        response.setVoltDiffValue(r.getValue());
                        response.setVoltDiffValueDiff(r.getValuediff());
                        response.setVoltDiffThreshold(new ArrayList<Object>() {{
                            add(r.getThresholdDown());
                            add(r.getThresholdUp());
                        }});
                        response.setVoltDiffLevel(r.getLevel());
                    } else if (r.getType().equals("内短路报警")) {
                        response.setInnershortAlarmTime(new SimpleDateFormat("yyyy-MM-dd").format(r.getAlarmtime()));
                        response.setInnershortValue(r.getValue());
                        response.setInnershortValueDiff(r.getValuediff());
                        response.setInnershortThreshold(new ArrayList<Object>() {{
                            add(r.getThresholdDown());
                            add(r.getThresholdUp());
                        }});
                        response.setInnershortLevel(r.getLevel());
                    }
                }
            }
            log.info(param.getSn() + "右侧结果返回");
            return ApiDTO.ok(response);
        } catch (Exception e) {
            throw new ApiRuntimeException(500, param.getSn() + e.toString());
        }
    }

    @ApiOperation(value = "获取内短路数据")
    @PostMapping("/getInner")
    public ApiDTO getInner(@RequestBody QueryParam param) {

        try {
            log.info(param.getSn() + "正在获取内短路数据");
            Response response = new Response();
            // 绘图信息
            ResultInfo byId = resultInfoService.getById(1);
            // 内短路绘图数据
            List<ResultInnershort> resultInnershorts = resultInnershortService.list(new QueryWrapper<ResultInnershort>().eq("sn", param.getSn()));
            List<String> innerShortDate = new ArrayList<>();
            Map<String, Object> innerShortData = new HashMap<>();
            List<List<Float>> innerShortDatas = Arrays.asList(new ArrayList<>(), new ArrayList<>(), new ArrayList<>(), new ArrayList<>(), new ArrayList<>(), new ArrayList<>(),
                    new ArrayList<>(), new ArrayList<>(), new ArrayList<>(), new ArrayList<>());
            for (ResultInnershort r : resultInnershorts) {
                innerShortDate.add(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(r.getTime()));
                innerShortDatas.get(0).add(r.getCell1());
                innerShortDatas.get(1).add(r.getCell2());
                innerShortDatas.get(2).add(r.getCell3());
                innerShortDatas.get(3).add(r.getCell4());
                innerShortDatas.get(4).add(r.getCell5());
                innerShortDatas.get(5).add(r.getCell6());
                innerShortDatas.get(6).add(r.getCell7());
                innerShortDatas.get(7).add(r.getCell8());
                innerShortDatas.get(8).add(r.getCell9());
                innerShortDatas.get(9).add(r.getCell10());
            }
            for (int i = 0; i < 10; i++) {
                innerShortData.put("cell_" + i, innerShortDatas.get(i));
            }
            response.setInnershortTime(innerShortDate);
            response.setInnershortY(innerShortDatas);
            log.info(param.getSn() + "内短路结果返回");
            return ApiDTO.ok(response);
        } catch (Exception e) {
            throw new ApiRuntimeException(500, e.toString());
        }
    }

    @ApiOperation(value = "获取电压离群数据")
    @PostMapping("/getVolt")
    public ApiDTO getVolt(@RequestBody QueryParam param) {

        try {
            log.info(param.getSn() + "正在获取电压离群数据");
            Response response = new Response();
            // 电压离群指数
            List<ResultVoltOutlier> resultVoltOutliers = resultVoltOutlierService.list(new QueryWrapper<ResultVoltOutlier>().eq("sn", param.getSn()));
            List<String> voltOutlierDate = new ArrayList<>();
            Map<String, Object> voltOutlierData = new HashMap<>();
            List<List<Float>> voltOutlierDatas = Arrays.asList(new ArrayList<>(), new ArrayList<>(), new ArrayList<>(), new ArrayList<>(), new ArrayList<>(), new ArrayList<>(),
                    new ArrayList<>(), new ArrayList<>(), new ArrayList<>(), new ArrayList<>());
            for (ResultVoltOutlier r : resultVoltOutliers) {
                voltOutlierDate.add(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(r.getTime()));
                voltOutlierDatas.get(0).add(r.getCell1());
                voltOutlierDatas.get(1).add(r.getCell2());
                voltOutlierDatas.get(2).add(r.getCell3());
                voltOutlierDatas.get(3).add(r.getCell4());
                voltOutlierDatas.get(4).add(r.getCell5());
                voltOutlierDatas.get(5).add(r.getCell6());
                voltOutlierDatas.get(6).add(r.getCell7());
                voltOutlierDatas.get(7).add(r.getCell8());
                voltOutlierDatas.get(8).add(r.getCell9());
                voltOutlierDatas.get(9).add(r.getCell10());
            }
            for (int i = 0; i < 10; i++) {
                voltOutlierData.put("cell_" + i, voltOutlierDatas.get(i));
            }
            response.setVoltoutlierTime(voltOutlierDate);
            response.setVoltoutlierY(voltOutlierDatas);
            log.info(param.getSn() + "电压离群结果返回");
            return ApiDTO.ok(response);
        } catch (Exception e) {
            throw new ApiRuntimeException(500, e.toString());
        }
    }

    @ApiOperation(value = "获取压差离群数据")
    @PostMapping("/getVoltdiff")
    public ApiDTO getVoltdiff(@RequestBody QueryParam param) {

        try {
            log.info(param.getSn() + "正在获取压差离群数据");
            Response response = new Response();
            // 压差离群指数
            List<ResultVoltdiffOutlier> resultVoltdiffOutliers = resultVoltdiffOutlierService.list(new QueryWrapper<ResultVoltdiffOutlier>().eq("sn", param.getSn()));
            List<String> voltdiffOutlierDate = new ArrayList<>();
            Map<String, Object> voltdiffOutlierData = new HashMap<>();
            List<List<Float>> voltdiffOutlierDatas = Arrays.asList(new ArrayList<>(), new ArrayList<>(), new ArrayList<>(), new ArrayList<>(), new ArrayList<>(), new ArrayList<>(),
                    new ArrayList<>(), new ArrayList<>(), new ArrayList<>(), new ArrayList<>());
            for (ResultVoltdiffOutlier r : resultVoltdiffOutliers) {
                voltdiffOutlierDate.add(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(r.getTime()));
                voltdiffOutlierDatas.get(0).add(r.getCell1());
                voltdiffOutlierDatas.get(1).add(r.getCell2());
                voltdiffOutlierDatas.get(2).add(r.getCell3());
                voltdiffOutlierDatas.get(3).add(r.getCell4());
                voltdiffOutlierDatas.get(4).add(r.getCell5());
                voltdiffOutlierDatas.get(5).add(r.getCell6());
                voltdiffOutlierDatas.get(6).add(r.getCell7());
                voltdiffOutlierDatas.get(7).add(r.getCell8());
                voltdiffOutlierDatas.get(8).add(r.getCell9());
                voltdiffOutlierDatas.get(9).add(r.getCell10());
            }
            for (int i = 0; i < 10; i++) {
                voltdiffOutlierData.put("cell_" + i, voltdiffOutlierDatas.get(i));
            }
            response.setVoltdiffoutlierTime(voltdiffOutlierDate);
            response.setVoltdiffoutlierY(voltdiffOutlierDatas);
            log.info(param.getSn() + "压差离群结果返回");
            return ApiDTO.ok(response);
        } catch (Exception e) {
            throw new ApiRuntimeException(500, e.toString());
        }
    }

    @ApiOperation(value = "获取soh数据")
    @PostMapping("/getSoh")
    public ApiDTO getSoh(@RequestBody QueryParam param) {

        try {
            log.info(param.getSn() + "正在获取soh数据");

            Response response = new Response();
            // soh 数据
//            List<ResultVoltdiffOutlier> resultVoltdiffOutliers = resultVoltdiffOutlierService.list(new QueryWrapper<ResultVoltdiffOutlier>().eq("sn", param.getSn()));
            List<SohResultDto> sohResults = sohResultMapper.getSohResult(param.getSn());
            List<String> sohDate = new ArrayList<>();
            List<Float> sohData = new ArrayList<>();
            for (SohResultDto r : sohResults) {
                sohDate.add(r.getTime());
                sohData.add(r.getSoh());
            }
            response.setSohTime(sohDate);
            response.setSohY(sohData);

            log.info(param.getSn() + "soh结果返回");
            return ApiDTO.ok(response);
        } catch (Exception e) {
            throw new ApiRuntimeException(500, e.toString());
        }
    }

    @ApiOperation(value = "获取sn列表")
    @PostMapping("/getSn")
    public ApiDTO getSn(@RequestBody QueryParam param) {
        List<String> snList = new ArrayList<>();
        List<ResultOriginData> resultOriginData = resultOriginDataService.list(new QueryWrapper<ResultOriginData>().eq("type", param.getTableId()).eq("batch_num", param.getBatchNum()));
        resultOriginData.forEach(p -> snList.add(p.getDeviceId()));
        return ApiDTO.ok(snList);
    }

}
